import * as ts from 'typescript';
/**
 * Get the type name of a given type.
 * @param typeChecker The context sensitive TypeScript TypeChecker.
 * @param type The type to get the name of.
 */
export declare function getTypeName(typeChecker: ts.TypeChecker, type: ts.Type): string;
//# sourceMappingURL=getTypeName.d.ts.map