import * as ts from 'typescript';
/**
 * Resolves the given node's type. Will resolve to the type's generic constraint, if it has one.
 */
export declare function getConstrainedTypeAtLocation(checker: ts.TypeChecker, node: ts.Node): ts.Type;
//# sourceMappingURL=getConstrainedTypeAtLocation.d.ts.map
